/*
  LostEnterCode
  by miniil
*/
#include <LiquidCrystal_I2C.h>
#include <Keypad.h>

const byte ROWS = 4;
const byte COLUMNS = 4;

const int L4 = 8; 
const int L3 = 9; 
const int L2 = 10; 
const int L1 = 11; 

const int C4 = 2; 
const int C3 = 3; 
const int C2 = 4; 
const int C1 = 5; 

//Key definition
char keys[ROWS][COLUMNS] = {
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'*', '0', '#', 'D'}
};

//Array rows and columns
byte pinRows[ROWS] = {L1, L2, L3, L4}; //connection for rows
byte pinColumns[COLUMNS] = {C1, C2, C3, C4}; //connection for columns

char key; //keep the key

//keypad object creation
Keypad pad = Keypad( makeKeymap(keys), pinRows, pinColumns, ROWS, COLUMNS );

//initialize the library with the numbers of the interface pins
LiquidCrystal_I2C lcd(0x27, 16, 2);

void setup() {
  Serial.begin(9600); //For debug
  lcd.init(); // initialize the lcd
  lcd.backlight(); // Turn on backlight
  delay(1000); //Little delay
}

//function to write text from right to left
void scrollInFromRight (int line, String str) 
{
  for (int j = 16; j >= 0; j--) 
  {
    lcd.setCursor(0, line);
    for (int k = 0; k <= 15; k++) 
    {
      lcd.print(" "); // Clear line
    }

    lcd.setCursor(j, line);
    lcd.print(str);
    delay(250);
  }
}

//function to test keypad
void testKeyPad()
{
  char key;
  String keyEntered = "";
  while (key != '#')
  {
    key = pad.getKey();
    if (key != NO_KEY && key != '#')
    {
      Serial.println(key);
      keyEntered += key;
      Serial.println(keyEntered);
      delay(300);
    }

  }

  Serial.println(keyEntered);
}

void loop() {
  scrollInFromRight(0, "SWAN STATION");
  delay(1000);
  
  int tps = 10;
  lcd.setCursor(5, 1);

  for (int tps = 10; tps > 0; tps--)
  {
    lcd.setCursor(0, 1);
    lcd.print("  "); //Clear 2 characters
    lcd.setCursor(0, 1);
    lcd.print(tps);
    delay(1000);
  }
    
  warning();  
}

bool warning()
{
  String trueLost = "4815162342";
  bool testCode = false;

  lcd.clear();
  lcd.setCursor(4, 0);
  lcd.print("WARNING");
  delay(2000);

  lcd.clear();
  lcd.setCursor(3, 0);
  lcd.print("KEEP KALM!");
  delay(2000);  

  while (testCode == false)
  {
    lcd.clear();
    lcd.setCursor(1, 0);
    lcd.print("ENTER THE CODE");
    lcd.setCursor(2, 1);
    lcd.print("THEN PRESS #");

    String receivedLost = getCode();
    Serial.println(receivedLost);
    delay(1000);

    lcd.clear();    
    lcd.print("ANALYSING...");
    lcd.setCursor(0, 1);
    lcd.print(receivedLost);
    delay(2000); 

    lcd.clear();
    if (receivedLost.equals(trueLost))
    {
      lcd.print("CODE CORRECT");
      testCode = true;
      delay(2000);
    }
    else
    {
      lcd.print("YOU FAIL !!!");
      lcd.setCursor(0, 1);
      lcd.print("TRY AGAIN");
      delay(2000);
    }    
  }
}

String getCode()
{
  char key;
  String keyEntered = "";
  while (key != '#')
  {
    key = pad.getKey();
    if (key != NO_KEY && key != '#')
    {
      keyEntered += key;
      delay(300);
    }

  }

  return keyEntered;
}
